Package: scrapper
Version: 1.7.0
Date: 2026-04-24
Authors@R: person("Aaron", "Lun", role=c("cre", "aut"), email="infinite.monkeys.with.keyboards@gmail.com")
Title: Bindings to C++ Libraries for Single-Cell Analysis
Description: 
    Implements R bindings to C++ code for analyzing single-cell (expression) data, mostly from various libscran libraries. 
    Each function performs an individual step in the single-cell analysis workflow, ranging from quality control to clustering and marker detection.
    Additional wrappers are provided for easy construction of end-to-end workflows involving Bioconductor objects like SingleCellExperiments.
License: MIT + file LICENSE
Imports: methods, Rcpp, beachmat (>= 2.25.1), S4Vectors, SparseArray,
        DelayedArray, BiocNeighbors (>= 1.99.0), parallel
Suggests: testthat, knitr, rmarkdown, BiocStyle, Matrix, IRanges,
        SummarizedExperiment, SingleCellExperiment, scRNAseq,
        org.Mm.eg.db, scater, igraph
LinkingTo: Rcpp, assorthead (>= 1.5.16), beachmat, BiocNeighbors,
        Rigraphlib
biocViews: Normalization, RNASeq, Software, GeneExpression,
        Transcriptomics, SingleCell, BatchEffect, QualityControl,
        DifferentialExpression, FeatureExtraction, PrincipalComponent,
        Clustering
SystemRequirements: C++17, GNU make
URL: https://github.com/libscran/scrapper
BugReports: https://github.com/libscran/scrapper/issues
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/scrapper
git_branch: devel
git_last_commit: 8005249
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 01:07:39 UTC; biocbuild
Author: Aaron Lun [cre, aut]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
