# Generated by roxygen2: do not edit by hand

export(LogNormalizedMatrix)
export(LogNormalizedMatrixSeed)
export(aggregateAcrossCells)
export(aggregateAcrossCells.se)
export(aggregateAcrossGenes)
export(aggregateAcrossGenes.se)
export(aggregateColData)
export(analyze)
export(analyze.se)
export(buildSnnGraph)
export(centerSizeFactors)
export(chooseHighlyVariableGenes)
export(choosePseudoCount)
export(chooseRnaHvgs.se)
export(clusterGraph)
export(clusterGraph.se)
export(clusterKmeans)
export(clusterKmeans.se)
export(combineFactors)
export(computeAdtQcMetrics)
export(computeBlockWeights)
export(computeClrm1Factors)
export(computeCrisprQcMetrics)
export(computeRnaQcMetrics)
export(computeRnaQcMetricsWithAltExps)
export(convertAnalyzeResults)
export(correctMnn)
export(correctMnn.se)
export(countGroupsByBlock)
export(filterAdtQcMetrics)
export(filterCrisprQcMetrics)
export(filterRnaQcMetrics)
export(fitVarianceTrend)
export(formatComputeAdtQcMetricsResult)
export(formatComputeCrisprQcMetricsResult)
export(formatComputeRnaQcMetricsResult)
export(formatModelGeneVariancesResult)
export(formatScoreMarkersResult)
export(getTestAdtData.se)
export(getTestCrisprData.se)
export(getTestRnaData.se)
export(modelGeneVariances)
export(normalizeAdtCounts.se)
export(normalizeCounts)
export(normalizeCrisprCounts.se)
export(normalizeRnaCounts.se)
export(previewMarkers)
export(quickAdtQc.se)
export(quickCrisprQc.se)
export(quickRnaQc.se)
export(reportGroupMarkerStatistics)
export(runAllNeighborSteps)
export(runAllNeighborSteps.se)
export(runPca)
export(runPca.se)
export(runTsne)
export(runTsne.se)
export(runUmap)
export(runUmap.se)
export(sanitizeSizeFactors)
export(scaleByNeighbors)
export(scaleByNeighbors.se)
export(scoreGeneSet)
export(scoreGeneSet.se)
export(scoreMarkers)
export(scoreMarkers.se)
export(subsampleByNeighbors)
export(suggestAdtQcThresholds)
export(suggestCrisprQcThresholds)
export(suggestRnaQcThresholds)
export(summarizeEffects)
export(testEnrichment)
export(tsnePerplexityToNeighbors)
exportClasses(LogNormalizedMatrix)
exportClasses(LogNormalizedMatrixSeed)
exportMethods(DelayedArray)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(extract_array)
exportMethods(extract_sparse_array)
exportMethods(initializeCpp)
exportMethods(is_sparse)
exportMethods(type)
import(methods)
importClassesFrom(DelayedArray,DelayedArray)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(S4Vectors,List)
importFrom(BiocNeighbors,AnnoyParam)
importFrom(BiocNeighbors,buildIndex)
importFrom(BiocNeighbors,defineBuilder)
importFrom(BiocNeighbors,findKNN)
importFrom(DelayedArray,DelayedArray)
importFrom(DelayedArray,extract_array)
importFrom(DelayedArray,new_DelayedArray)
importFrom(DelayedArray,t)
importFrom(DelayedArray,type)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,I)
importFrom(S4Vectors,List)
importFrom(S4Vectors,cbind)
importFrom(S4Vectors,make_zero_col_DFrame)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,setValidity2)
importFrom(SparseArray,extract_sparse_array)
importFrom(SparseArray,is_sparse)
importFrom(beachmat,getExecutor)
importFrom(beachmat,initializeCpp)
importFrom(beachmat,tatami.column.sums)
importFrom(beachmat,tatami.dim)
importFrom(beachmat,tatami.subset)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(utils,head)
useDynLib(scrapper)
